@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -74,7 +74,7 @@ class Cleaner:
         """Register 'action' (instance of class Action) to be executed
         for ''option_id'.  The actions must implement list_files and
         other_cleanup()"""
-        self.actions += ((option_id, action), )
+        self.actions.append((option_id, action))
 
     def add_option(self, option_id, name, description):
         """Register option (such as 'cache')"""
@@ -82,7 +82,7 @@ class Cleaner:
 
     def add_running(self, detection_type, pathname):
         """Add a way to detect this program is currently running"""
-        self.running += ((detection_type, pathname), )
+        self.running.append((detection_type, pathname))
 
     def auto_hide(self):
         """Return boolean whether it is OK to automatically hide this
@@ -156,12 +156,9 @@ class Cleaner:
         for running in self.running:
             test = running[0]
             pathname = running[1]
-            if 'exe' == test and 'posix' == os.name:
-                if Unix.is_running(pathname):
-                    logger.debug("process '%s' is running", pathname)
-                    return True
-            elif 'exe' == test and 'nt' == os.name:
-                if Windows.is_process_running(pathname):
+            if 'exe' == test:
+                if ('posix' == os.name and Unix.is_running(pathname)) or \
+                   ('nt' == os.name and Windows.is_process_running(pathname)):
                     logger.debug("process '%s' is running", pathname)
                     return True
             elif 'pathname' == test:
@@ -202,8 +199,8 @@ class OpenOfficeOrg(Cleaner):
         # reference: http://katana.oooninja.com/w/editions_of_openoffice.org
         if 'posix' == os.name:
             self.prefixes = ["~/.ooo-2.0", "~/.openoffice.org2",
-                             "~/.openoffice.org2.0", "~/.openoffice.org/3"]
-            self.prefixes += ["~/.ooo-dev3"]
+                             "~/.openoffice.org2.0", "~/.openoffice.org/3",
+                             "~/.ooo-dev3"]
         if 'nt' == os.name:
             self.prefixes = [
                 "$APPDATA\\OpenOffice.org\\3", "$APPDATA\\OpenOffice.org2"]
@@ -382,10 +379,10 @@ class System(Cleaner):
                      '~/.kde2/share/applnk']
 
         if 'posix' == os.name and 'desktop_entry' == option_id:
-            for dirname in menu_dirs:
-                for filename in [fn for fn in children_in_directory(dirname, False)
-                                 if fn.endswith('.desktop')]:
-                    if Unix.is_broken_xdg_desktop(filename):
+            for path in menu_dirs:
+                dirname = os.path.expanduser(path)
+                for filename in children_in_directory(dirname, False):
+                    if filename.endswith('.desktop') and Unix.is_broken_xdg_desktop(filename):
                         yield Command.Delete(filename)
 
         # unwanted locales
@@ -614,14 +611,15 @@ class System(Cleaner):
             '^/tmp/.truecrypt_aux_mnt.*/(control|volume)$',
             '^/tmp/.vbox-[^/]+-ipc/lock$',
             '^/tmp/.wine-[0-9]+/server-.*/lock$',
-            '^/tmp/gconfd-[^/]+/lock/ior$',
             '^/tmp/fsa/',  # fsarchiver
+            '^/tmp/gconfd-[^/]+/lock/ior$',
             '^/tmp/kde-',
             '^/tmp/kdesudo-',
             '^/tmp/ksocket-',
             '^/tmp/orbit-[^/]+/bonobo-activation-register[a-z0-9-]*.lock$',
             '^/tmp/orbit-[^/]+/bonobo-activation-server-[a-z0-9-]*ior$',
             '^/tmp/pulse-[^/]+/pid$',
+            '^/tmp/xauth',
             '^/var/tmp/kdecache-',
             '^' + os.path.expanduser('~/.cache/wallpaper/'),
             # Flatpak mount point
