@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2019 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -20,9 +20,11 @@
 
 """
 Command design pattern implementation for cleaning
-"""
 
-from __future__ import absolute_import
+Standard clean up commands are Delete, Truncate and Shred. Everything
+else is counted as special commands: run any external process, edit
+JSON or INI file, delete registry key, edit SQLite3 database, etc.
+"""
 
 from bleachbit import _
 from bleachbit import FileUtilities
@@ -31,13 +33,13 @@ import logging
 import os
 import types
 
-from sqlite3 import DatabaseError
-
 if 'nt' == os.name:
     import bleachbit.Windows
 else:
     from bleachbit.General import WindowsError
 
+logger = logging.getLogger(__name__)
+
 
 def whitelist(path):
     """Return information that this file was whitelisted"""
@@ -84,7 +86,7 @@ class Delete:
                 FileUtilities.delete(self.path, self.shred)
             except WindowsError as e:
                 # WindowsError: [Error 32] The process cannot access the file because it is being
-                # used by another process: u'C:\\Documents and
+                # used by another process: 'C:\\Documents and
                 # Settings\\username\\Cookies\\index.dat'
                 if 32 != e.winerror and 5 != e.winerror:
                     raise
@@ -150,21 +152,23 @@ class Function:
                             # If tuple, then display progress.
                             yield func_ret
                 # either way, func_ret should be an integer
-                assert isinstance(func_ret, (int, long))
+                assert isinstance(func_ret, int)
                 ret['size'] = func_ret
             else:
                 if os.path.isdir(self.path):
                     raise RuntimeError('Attempting to run file function %s on directory %s' %
-                                       (self.func.func_name, self.path))
+                                       (self.func.__name__, self.path))
                 # Function takes a path.  We check the size.
                 oldsize = FileUtilities.getsize(self.path)
+
+                from sqlite3 import DatabaseError
                 try:
                     self.func(self.path)
                 except DatabaseError as e:
                     if -1 == e.message.find('file is encrypted or is not a database') and \
                        -1 == e.message.find('or missing database'):
                         raise
-                    logging.getLogger(__name__).exception(e.message)
+                    logger.exception(e.message)
                     return
                 try:
                     newsize = FileUtilities.getsize(self.path)
@@ -284,8 +288,8 @@ class Truncate(Delete):
             'path': self.path,
             'size': FileUtilities.getsize(self.path)}
         if really_delete:
-            f = open(self.path, 'wb')
-            f.truncate(0)
+            with open(self.path, 'w') as f:
+                f.truncate(0)
         yield ret
 
 
@@ -304,20 +308,21 @@ class Winreg:
     def execute(self, really_delete):
         """Execute the Windows registry cleaner"""
         if 'nt' != os.name:
-            raise StopIteration
+            return
         _str = None  # string representation
         ret = None  # return value meaning 'deleted' or 'delete-able'
         if self.valuename:
             _str = '%s<%s>' % (self.keyname, self.valuename)
             ret = bleachbit.Windows.delete_registry_value(self.keyname,
-                                                self.valuename, really_delete)
+                                                          self.valuename, really_delete)
         else:
-            ret = bleachbit.Windows.delete_registry_key(self.keyname, really_delete)
+            ret = bleachbit.Windows.delete_registry_key(
+                self.keyname, really_delete)
             _str = self.keyname
         if not ret:
             # Nothing to delete or nothing was deleted.  This return
             # makes the auto-hide feature work nicely.
-            raise StopIteration
+            return
 
         ret = {
             'label': _('Delete registry key'),
