@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -138,6 +138,10 @@ def open_files():
             # happens, for example, when link points to
             # '/etc/password\x00 (deleted)'
             continue
+        except PermissionError:
+            # /proc/###/fd/0 with systemd
+            # https://github.com/bleachbit/bleachbit/issues/1515
+            continue
         else:
             yield target
 
@@ -928,6 +932,9 @@ def wipe_path(pathname, idle=False):
         return 1, done_percent, remaining_seconds
 
     logger.debug(_("Wiping path: %s") % pathname)
+    if not os.path.isdir(pathname):
+        logger.error(_("Path to wipe must be an existing directory: %s"), pathname)
+        return
     files = []
     total_bytes = 0
     start_free_bytes = free_space(pathname)
