@@ -2,7 +2,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -26,6 +26,11 @@ from bleachbit import GuiBasic
 from bleachbit import Cleaner, FileUtilities
 from bleachbit import _, APP_NAME, appicon_path, portable_mode, windows10_theme_path
 from bleachbit.Options import options
+
+# Now that the configuration is loaded, honor the debug preference there.
+from bleachbit.Log import set_root_log_level
+set_root_log_level(options.get('debug'))
+
 from bleachbit.GuiPreferences import PreferencesDialog
 from bleachbit.Cleaner import backends, register_cleaners
 import bleachbit
@@ -136,29 +141,24 @@ class Bleachbit(Gtk.Application):
             import atexit
             atexit.register(Windows.cleanup_nonce)
 
-            # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
-            # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
-            # https://www.bleachbit.org/forum/074-fails-errors
-            try:
-                import sqlite3
-            except ImportError:
-                logger.exception(
-                    _("Error loading the SQLite module: the antivirus software may be blocking it."))
+        from bleachbit.General import startup_check
+        startup_check()
 
     def _init_windows_misc(self, auto_exit, shred_paths, uac):
         application_id_suffix = ''
         is_context_menu_executed = auto_exit and shred_paths
-        if os.name == 'nt':
-            if Windows.elevate_privileges(uac):
-                # privileges escalated in other process
-                sys.exit(0)
+        if not os.name == 'nt':
+            return ''
+        if Windows.elevate_privileges(uac):
+            # privileges escalated in other process
+            sys.exit(0)
 
-            if is_context_menu_executed:
-                # When we have a running application and executing the Windows
-                # context menu command we start a new process with new application_id.
-                # That is because the command line arguments of the context menu command
-                # are not passed to the already running instance.
-                application_id_suffix = 'ContextMenuShred'
+        if is_context_menu_executed:
+            # When we have a running application and executing the Windows
+            # context menu command we start a new process with new application_id.
+            # That is because the command line arguments of the context menu command
+            # are not passed to the already running instance.
+            application_id_suffix = 'ContextMenuShred'
         return application_id_suffix
 
     def build_app_menu(self):
@@ -301,8 +301,8 @@ class Bleachbit(Gtk.Application):
         GUI.update_log_level(self._window)
 
     def get_about_dialog(self):
-        dialog = Gtk.AboutDialog(comments='Program to clean unnecessary files',
-                                 copyright='Copyright (C) 2008-2021 Andrew Ziem',
+        dialog = Gtk.AboutDialog(comments=_("Program to clean unnecessary files"),
+                                 copyright='Copyright (C) 2008-2023 Andrew Ziem',
                                  program_name=APP_NAME,
                                  version=bleachbit.APP_VERSION,
                                  website=bleachbit.APP_URL,
@@ -373,6 +373,8 @@ class Bleachbit(Gtk.Application):
         if not self._window:
             self._window = GUI(
                 application=self, title=APP_NAME, auto_exit=self._auto_exit)
+        if 'nt' == os.name:
+            Windows.check_dll_hijacking(self._window)
         self._window.present()
         if self._shred_paths:
             GLib.idle_add(GUI.shred_paths, self._window, self._shred_paths, priority=GObject.PRIORITY_LOW)
@@ -435,9 +437,16 @@ class TreeInfoModel:
                                                               self.on_row_changed)
 
     def sort_func(self, model, iter1, iter2, _user_data):
-        """Sort the tree by the display name"""
-        value1 = model[iter1][0].lower()
-        value2 = model[iter2][0].lower()
+        """Sort the tree by the id
+
+        Index 0 is the display name
+        Index 2 is the ID (e.g., cookies, vacuum).
+
+        Sorting by ID is functionally important, so that vacuuming is done
+        last, even for other languages. See https://github.com/bleachbit/bleachbit/issues/441
+        """
+        value1 = model[iter1][2].lower()
+        value2 = model[iter2][2].lower()
         if value1 == value2:
             return 0
         if value1 > value2:
@@ -513,7 +522,9 @@ class TreeDisplayModel:
             if warning:
                 resp = GuiBasic.message_dialog(parent_window,
                                                msg,
-                                               Gtk.MessageType.WARNING, Gtk.ButtonsType.OK_CANCEL)
+                                               Gtk.MessageType.WARNING,
+                                               Gtk.ButtonsType.OK_CANCEL,
+                                               _('Confirm'))
                 if Gtk.ResponseType.OK != resp:
                     # user cancelled, so don't toggle option
                     return
@@ -593,7 +604,7 @@ class GUI(Gtk.ApplicationWindow):
 
         font_conf_file = Windows.get_font_conf_file()
         if not os.path.exists(font_conf_file):
-            logger.error('No fonts.conf file')
+            logger.error('No fonts.conf file {}'.format(font_conf_file))
             return
 
         has_cache = Windows.has_fontconfig_cache(font_conf_file)
@@ -605,6 +616,11 @@ class GUI(Gtk.ApplicationWindow):
             return GuiBasic.delete_confirmation_dialog(self, mention_preview, shred_settings=shred_settings)
         return True
 
+    def destroy(self):
+        """Prevent textbuffer usage during UI destruction"""
+        self.textbuffer = None
+        super(GUI, self).destroy()
+
     def get_preferences_dialog(self):
         return PreferencesDialog(
             self,
@@ -640,6 +656,9 @@ class GUI(Gtk.ApplicationWindow):
 
     def append_text(self, text, tag=None, __iter=None, scroll=True):
         """Add some text to the main log"""
+        if self.textbuffer is None:
+            # textbuffer was destroyed.
+            return
         if not __iter:
             __iter = self.textbuffer.get_end_iter()
         if tag:
@@ -650,7 +669,7 @@ class GUI(Gtk.ApplicationWindow):
         # through the idle loop, it may only scroll most of the way
         # as seen on Ubuntu 9.04 with Italian and Spanish.
         if scroll:
-            GLib.idle_add(lambda:
+            GLib.idle_add(lambda: self.textbuffer is not None and
                           self.textview.scroll_mark_onscreen(
                               self.textbuffer.get_insert()))
 
@@ -747,7 +766,9 @@ class GUI(Gtk.ApplicationWindow):
         if not operations:  # empty
             GuiBasic.message_dialog(self,
                                     _("You must select an operation"),
-                                    Gtk.MessageType.WARNING, Gtk.ButtonsType.OK)
+                                    Gtk.MessageType.ERROR,
+                                    Gtk.ButtonsType.OK,
+                                    _('Error'))
             return
         try:
             self.set_sensitive(False)
@@ -865,6 +886,10 @@ class GUI(Gtk.ApplicationWindow):
                     _('Run BleachBit with administrator privileges to improve the accuracy of overwriting the contents of files.'))
                 self.append_text('\n')
 
+        if 'windowsapps' in sys.executable.lower():
+            self.append_text(
+                _('There is no official version of BleachBit on the Microsoft Store. Get the genuine version at https://www.bleachbit.org where it is always free of charge.') + '\n', 'error')
+
         # remove from idle loop (see GObject.idle_add)
         return False
 
@@ -1081,8 +1106,6 @@ class GUI(Gtk.ApplicationWindow):
 
     def on_window_state_event(self, widget, event):
         # save window state
-        fullscreen = event.new_window_state & Gdk.WindowState.FULLSCREEN != 0
-        options.set("window_fullscreen", fullscreen, commit=False)
         maximized = event.new_window_state & Gdk.WindowState.MAXIMIZED != 0
         options.set("window_maximized", maximized, commit=False)
         return False
@@ -1119,9 +1142,7 @@ class GUI(Gtk.ApplicationWindow):
                     monitor_num, (g.x, g.y), (g.width, g.height), (r.x, r.y), (r.width, r.height)))
                 self.move(r.x, r.y)
                 self.resize(r.width, r.height)
-        if options.get("window_fullscreen"):
-            self.fullscreen()
-        elif options.get("window_maximized"):
+        if options.get("window_maximized"):
             self.maximize()
 
     def set_windows10_theme(self):
