@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -45,10 +45,9 @@ def boolstr_to_bool(value):
 def getText(nodelist):
     """Return the text data in an XML node
     http://docs.python.org/library/xml.dom.minidom.html"""
-    rc = ""
-    for node in nodelist:
-        if node.nodeType == node.TEXT_NODE:
-            rc = rc + node.data
+    rc = "".join(
+        node.data for node in nodelist if node.nodeType == node.TEXT_NODE
+    )
     return rc
 
 
@@ -141,9 +140,8 @@ def run_external(args, stdout=None, env=None, clean_env=True):
         # strings in the output.
         # https://github.com/bleachbit/bleachbit/issues/167
         # https://github.com/bleachbit/bleachbit/issues/168
-        keep_env = ('PATH', 'HOME', 'LD_LIBRARY_PATH', 'TMPDIR')
-        env = dict((key, value)
-                   for key, value in os.environ.items() if key in keep_env)
+        keep_env = ('PATH', 'HOME', 'LD_LIBRARY_PATH', 'TMPDIR', 'BLEACHBIT_TEST_OPTIONS_DIR')
+        env = {key: value for key, value in os.environ.items() if key in keep_env}
         env['LANG'] = 'C'
         env['LC_ALL'] = 'C'
     p = subprocess.Popen(args, stdout=stdout,
@@ -160,6 +158,24 @@ def run_external(args, stdout=None, env=None, clean_env=True):
             str(out[1], encoding=encoding) if out[1] else '')
 
 
+def startup_check():
+    """At application startup, check some things are okay."""
+
+    if 'nt' == os.name:
+        from bleachbit.Windows import check_dll_hijacking
+        check_dll_hijacking()
+
+    # BitDefender false positive.  BitDefender didn't mark BleachBit as infected or show
+    # anything in its log, but sqlite would fail to import unless BitDefender was in "game mode."
+    # https://www.bleachbit.org/forum/074-fails-errors
+    from bleachbit import ModuleNotFoundError
+    try:
+        from sqlite3 import SQLITE_OK
+    except (ModuleNotFoundError, ImportError):
+        logger.exception(
+            'The sqlite3 module could not be loaded. On Windows, the antivirus may be blocking it. On FreeBSD, install the package databases/py-sqlite3.')
+
+
 def sudo_mode():
     """Return whether running in sudo mode"""
     if not sys.platform.startswith('linux'):
