@@ -3,7 +3,7 @@
 
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -102,7 +102,10 @@ class PreferencesDialog:
             self.cb_set_windows10_theme()
         if 'debug' == path:
             from bleachbit.Log import set_root_log_level
-            set_root_log_level()
+            set_root_log_level(options.get('debug'))
+        if 'kde_shred_menu_option' == path:
+            from bleachbit.DesktopMenuOptions import install_kde_service_menu_file
+            install_kde_service_menu_file()
 
     def __general_page(self):
         """Return a widget containing the general page"""
@@ -209,6 +212,12 @@ class PreferencesDialog:
         cb_debug.connect('toggled', self.__toggle_callback, 'debug')
         vbox.pack_start(cb_debug, False, True, 0)
 
+        # KDE context menu shred option
+        cb_kde_shred_menu_option = Gtk.CheckButton(label=_("Add shred context menu option (KDE Plasma specific)"))
+        cb_kde_shred_menu_option.set_active(options.get("kde_shred_menu_option"))
+        cb_kde_shred_menu_option.connect('toggled', self.__toggle_callback, 'kde_shred_menu_option')
+        vbox.pack_start(cb_kde_shred_menu_option, False, True, 0)
+
         return vbox
 
     def __drives_page(self):
@@ -440,7 +449,7 @@ class PreferencesDialog:
             # TRANSLATORS: "Paths" is used generically to refer to both files
             # and folders
             notice = Gtk.Label(
-                label=_("Theses paths will not be deleted or modified."))
+                label=_("These paths will not be deleted or modified."))
         elif LOCATIONS_CUSTOM == page_type:
             notice = Gtk.Label(
                 label=_("These locations can be selected for deletion."))
