@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2020 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -54,6 +54,7 @@ def update_winapp2(url, hash_expected, append_text, cb_success):
                 return
         delete_current = True
     # download update
+    # FIXME: refactor this to share code with bleachbit.Chaff.download_url_to_fn()
     opener = build_opener()
     opener.addheaders = [('User-Agent', user_agent())]
     doc = opener.open(fullurl=url, timeout=20).read()
@@ -84,7 +85,7 @@ def user_agent():
         __os = platform.uname()[3][
             0:3]  # 5.1 = Windows XP, 6.0 = Vista, 6.1 = 7
     elif sys.platform.startswith('linux'):
-        dist = platform.dist()
+        dist = platform.linux_distribution()
         # example: ('fedora', '11', 'Leonidas')
         # example: ('', '', '') for Arch Linux
         if 0 < len(dist[0]):
@@ -148,17 +149,40 @@ def update_dialog(parent, updates):
     return False
 
 
+def get_ip_for_url(url):
+    """Given an https URL, return the IP address"""
+    if not url:
+        return '(no URL)'
+    url_split = url.split('/')
+    if len(url_split) < 3:
+        return '(bad URL)'
+    hostname = url.split('/')[2]
+    import socket
+    try:
+        ip_address = socket.gethostbyname(hostname)
+    except socket.gaierror:
+        return '(socket.gaierror)'
+    return ip_address
+
+
 def check_updates(check_beta, check_winapp2, append_text, cb_success):
     """Check for updates via the Internet"""
     opener = build_opener()
     socket.setdefaulttimeout(bleachbit.socket_timeout)
     opener.addheaders = [('User-Agent', user_agent())]
-    import encodings.idna # https://github.com/bleachbit/bleachbit/issues/760
+    import encodings.idna  # https://github.com/bleachbit/bleachbit/issues/760
+    url = bleachbit.update_check_url
+    if 'windowsapp' in sys.executable.lower():
+        url += '?windowsapp=1'
     try:
-        handle = opener.open(bleachbit.update_check_url)
+        handle = opener.open(url)
     except URLError as e:
         logger.error(
             _('Error when opening a network connection to check for updates. Please verify the network is working and that a firewall is not blocking this application. Error message: {}').format(e))
+        logger.debug('URL {} has IP address {}'.format(
+            url, get_ip_for_url(url)))
+        if hasattr(e, 'headers'):
+            logger.debug(e.headers)
         return ()
     doc = handle.read()
     try:
