@@ -1,7 +1,7 @@
 # vim: ts=4:sw=4:expandtab
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -177,7 +177,7 @@ class CLITestCase(common.BleachbitTestCase):
             FileUtilities.delete(filename)
             self.assertExists(filename)
             operations = args_to_operations(['system.tmp'], False, False)
-            preview_or_clean(operations, True)
+            preview_or_clean(operations, True, quiet=True)
             FileUtilities.delete = save_delete
             self.assertIn(filename, deleted_paths,
                           "%s not found deleted" % filename)
