@@ -2,7 +2,7 @@
 # -*- coding: UTF-8 -*-
 
 # BleachBit
-# Copyright (C) 2008-2021 Andrew Ziem
+# Copyright (C) 2008-2023 Andrew Ziem
 # https://www.bleachbit.org
 #
 # This program is free software: you can redistribute it and/or modify
@@ -26,7 +26,6 @@ Test case for module Unix
 from tests import common
 from bleachbit.Unix import *
 
-import io
 import mock
 import os
 import sys
@@ -310,3 +309,36 @@ root               531   0.0  0.0  2501712    588   ??  Ss   20May16   0:02.40 s
             0, 'Remove  112 Packages\nFreed space: 299 M\n', 'stderr')
         bytes_freed = dnf_autoremove()
         self.assertEqual(bytes_freed, 299000000)
+
+    @common.skipIfWindows
+    def test_is_linux_display_protocol_wayland(self):
+        display_protocol = None
+        def side_effect_func(value):
+            if len(value) == 1:
+                return (0, 'SESSION  UID USER   SEAT  TTY \n      2 1000 debian seat0 tty2\n\n1 sessions listed.\n', '')
+            elif len(value) > 1:
+                return (0, 'Type={}\n'.format(display_protocol), '')
+            assert(False) # should never reach here
+
+        for display_protocol, assert_method in [['wayland', self.assertTrue], ['donotexist', self.assertFalse]]:
+            with mock.patch('bleachbit.General.run_external') as mock_run_external:
+                mock_run_external.side_effect = side_effect_func
+                is_wayland = is_linux_display_protocol_wayland()
+            self.assertIsInstance(is_wayland, bool)
+            assert_method(is_wayland)
+
+    @common.skipIfWindows
+    def test_xhost_autorized_root(self):
+        error_code = None
+        def side_effect_func(*args, **kwargs):
+            self.assertEqual(args[0][0], 'xhost')
+            return (error_code,
+                    '',
+                    '')
+
+        for error_code, expected_root_autorized in [[1, True], [0, False]]:
+            with mock.patch('bleachbit.General.run_external') as mock_run_external:
+                mock_run_external.side_effect = side_effect_func
+                root_autorized = root_is_not_allowed_to_X_session()
+            self.assertIsInstance(root_autorized, bool)
+            self.assertEqual(expected_root_autorized, root_autorized)
